@@ -17,9 +17,6 @@
 #
 #
 
-import block2
-from block2 import *
-from block2.su2 import *
 from pyscf import lib, ao2mo, gto, mcscf, tools, symm
 import numpy as np
 
@@ -28,9 +25,9 @@ try:
 except ImportError:
     from dmrgscf import DMRGCI as _DMRGCI
 
-class NEVPT(lib.StreamObject):
+class MRPT(lib.StreamObject):
 
-    def __init__(self, mc, mp_order=2, ci_order=2):
+    def __init__(self, mc, mp_order=2, ci_order=2, theory='nevpt', spin_adapted=True):
         self._casci = mc
         self._scf = mc._scf
         self.mol = self._scf.mol
@@ -40,23 +37,42 @@ class NEVPT(lib.StreamObject):
         self.mp_order = mp_order
         self.ci_order = ci_order
         self.dmrg_args = {
-            "startM": 250, "maxM": 500, "schedule": "default",
-            "sweep_tol": 1E-6, "cutoff": 1E-14,
+            "startM": 250, "maxM": 1800, "schedule": "default",
+            "sweep_tol": 1E-8, "cutoff": 1E-16,
             "memory": lib.param.MAX_MEMORY * 1E6
         }
+        self.theory = theory
+        self.scratch = './nodex'
+        self.spin_adapted = spin_adapted
 
     @property
     def e_tot(self):
-        return np.asarray(self.e_corr) + self._scf.e_tot
+        return np.asarray(self.e_corr) + self._casci.e_tot
 
-    def kernel(self, mo_coeff=None):
+    def kernel(self, mo_coeff=None, write_fd=None, write_conf=None):
 
         if mo_coeff is None:
-            mo_coeff = self._scf.mo_coeff
+            mo_coeff = self._casci.mo_coeff
+
+        from block2 import Random, Global, Threading, FCIDUMP, VectorUInt8, init_memory, VectorInt
+        from block2 import ThreadingTypes, SeqTypes, DoubleFPCodec, QCTypes, OpNamesSet, OpNames
+        from block2 import SU2, SZ, VectorUBond, VectorDouble, NoiseTypes, DyallFCIDUMP, FinkFCIDUMP
+        if self.spin_adapted:
+            SX = SU2
+            from block2.su2 import CSFBigSite, SimplifiedBigSite, RuleQC, HamiltonianQCBigSite
+            from block2.su2 import CASCIMPSInfo, MPOQC, SimplifiedMPO, IdentityAddedMPO
+            from block2.su2 import MovingEnvironment, DMRGBigSite, NoTransposeRule, MPSInfo, MPS
+            from block2.su2 import LinearBigSite
+        else:
+            SX = SZ
+            from block2.sz import SCIFockBigSite, SimplifiedBigSite, RuleQC, HamiltonianQCBigSite
+            from block2.sz import CASCIMPSInfo, MPOQC, SimplifiedMPO, IdentityAddedMPO
+            from block2.sz import MovingEnvironment, DMRGBigSite, NoTransposeRule, MPSInfo, MPS
+            from block2.sz import LinearBigSite
 
         # Global
         Random.rand_seed(123456)
-        scratch = './nodex'
+        scratch = self.scratch
         n_threads = lib.num_threads()
         fcidump_tol = 1E-13
         memory = self.dmrg_args["memory"]
@@ -79,9 +95,12 @@ class NEVPT(lib.StreamObject):
         
         if self.mol.groupname.lower() == 'c1':
             orb_sym = [0] * self.mol.nao
+            orb_sym_fd = [1] * self.mol.nao
         else:
-            orb_sym = irrep_name2id(symm.label_orb_symm(
-                self.mol, self.mol.irrep_name, self.mol.symm_orb, mo_coeff))
+            from pyscf import symm, tools
+            orb_sym_str = symm.label_orb_symm(self.mol, self.mol.irrep_name, self.mol.symm_orb, mo_coeff)
+            orb_sym = [symm.irrep_name2id(self.mol.groupname, i) for i in orb_sym_str]
+            orb_sym_fd = [tools.fcidump.ORBSYM_MAP[self.mol.groupname][i] for i in orb_sym]
 
         # FCIDUMP
         h1e = mo_coeff.T @ self._scf.get_hcore() @ mo_coeff
@@ -93,6 +112,7 @@ class NEVPT(lib.StreamObject):
         na, nb = self.mol.nelec
         fcidump = FCIDUMP()
         fcidump.initialize_su2(self.mol.nao, na + nb, abs(na - nb), 1, e_core, h1e, g2e)
+        fcidump.orb_sym = VectorUInt8(orb_sym_fd)
         error = fcidump.symmetrize(VectorUInt8(orb_sym))
         if self.verbose >= 5:
             print('symm error = ', error)
@@ -104,14 +124,47 @@ class NEVPT(lib.StreamObject):
         n_inactive = mc.ncore
         n_external = self.n_orbs - mc.ncas - mc.ncore
         assert n_inactive + n_external <= n_orbs
-        big_left_orig = CSFBigSite(n_inactive, self.ci_order, False,
-            fcidump, VectorUInt8(orb_sym[:n_inactive]), max(min(self.verbose - 4, 3), 0))
-        big_right_orig = CSFBigSite(n_external, self.ci_order, True,
-            fcidump, VectorUInt8(orb_sym[-n_external:]), max(min(self.verbose - 4, 3), 0))
+
+        if write_fd is not None:
+            fcidump.write(write_fd)
+        
+        if write_conf is not None:
+            with open(write_conf, "w") as conf:
+                conf.write("""
+                    sym %s
+                    orbitals %s
+
+                    schedule default
+                    maxM %d
+                    maxiter %d
+
+                    %s %d %d %d
+                    noreorder
+                """ % (
+                    self.mol.groupname.lower(),
+                    write_fd if write_fd is not None else "FCIDUMP",
+                    self.dmrg_args["maxM"],
+                    self.dmrg_args["maxiter"],
+                    "nevpt2sd" if self.theory == "nevpt" else "mrrept2sd",
+                    n_inactive, n_orbs - n_inactive - n_external, n_external
+                ))
+            self.e_corr = 0.0
+            return None
+
+        if self.spin_adapted:
+            big_left_orig = CSFBigSite(n_inactive, self.ci_order, False,
+                fcidump, VectorUInt8(orb_sym[:n_inactive]), max(min(self.verbose - 4, 3), 0))
+            big_right_orig = CSFBigSite(n_external, self.ci_order, True,
+                fcidump, VectorUInt8(orb_sym[-n_external:]), max(min(self.verbose - 4, 3), 0))
+        else:
+            poccl = SCIFockBigSite.ras_space(False, n_inactive, *[self.ci_order] * 3, VectorInt([]))
+            poccr = SCIFockBigSite.ras_space(True, n_external, *[self.ci_order] * 3, VectorInt([]))
+            big_left_orig = SCIFockBigSite(n_orbs, n_inactive, False, fcidump, VectorUInt8(orb_sym), poccl, self.verbose > 4)
+            big_right_orig = SCIFockBigSite(n_orbs, n_external, True, fcidump, VectorUInt8(orb_sym), poccr, self.verbose > 4)
         big_left = SimplifiedBigSite(big_left_orig, RuleQC())
         big_right = SimplifiedBigSite(big_right_orig, RuleQC())
-        vacuum = SU2(0)
-        target = SU2(na + nb, abs(na - nb), 0)
+        vacuum = SX(0)
+        target = SX(na + nb, abs(na - nb), 0)
         hamil = HamiltonianQCBigSite(vacuum, n_orbs, VectorUInt8(orb_sym), fcidump,
             big_left, big_right)
         n_sites = hamil.n_sites
@@ -139,6 +192,7 @@ class NEVPT(lib.StreamObject):
         # MPO
         mpo = MPOQC(hamil, QCTypes.NC)
         mpo = SimplifiedMPO(mpo, RuleQC(), True, True, OpNamesSet((OpNames.R, OpNames.RD)))
+        mpo = IdentityAddedMPO(mpo)
 
         # DMRG
         me = MovingEnvironment(mpo, mps, mps, "DMRG")
@@ -157,9 +211,9 @@ class NEVPT(lib.StreamObject):
         ener = dmrg.solve(len(bond_dims), forward, sweep_tol)
 
         self.converged = True
-        self.e_corr = ener - self._scf.e_tot
+        self.e_corr = 0
 
-        lib.logger.note(self, 'E(NEVPT1) = %.16g  E_corr = %.16g', self.e_tot, self.e_corr)
+        lib.logger.note(self, 'E(%s1) = %.16g  E_corr = %.16g', self.theory.upper(), self.e_tot, self.e_corr)
 
         self.mps = mps
 
@@ -167,10 +221,13 @@ class NEVPT(lib.StreamObject):
             return self.e_corr, mps
 
         # FCIDUMP
-        dm1 = self.make_rdm1(mps).copy(order='C')
-        fd_dyall = DyallFCIDUMP(fcidump, n_inactive, n_external)
-        fd_dyall.initialize_from_1pdm_su2(dm1)
-        error = fd_dyall.symmetrize(VectorUInt8(orb_sym))
+        if self.theory == 'nevpt':
+            dm1 = self.make_rdm1(mps).copy(order='C')
+            fd_zero = DyallFCIDUMP(fcidump, n_inactive, n_external)
+            fd_zero.initialize_from_1pdm_su2(dm1)
+        else:
+            fd_zero = FinkFCIDUMP(fcidump, n_inactive, n_external)
+        error = fd_zero.symmetrize(VectorUInt8(orb_sym))
         if self.verbose >= 5:
             print('symm error = ', error)
         
@@ -179,26 +236,34 @@ class NEVPT(lib.StreamObject):
         big_right = SimplifiedBigSite(big_right_orig, NoTransposeRule(RuleQC()))
         hamil = HamiltonianQCBigSite(vacuum, n_orbs, VectorUInt8(orb_sym), fcidump,
             big_left, big_right)
-        big_left_orig = CSFBigSite(n_inactive, self.ci_order, False,
-            fd_dyall, VectorUInt8(orb_sym[:n_inactive]), max(min(self.verbose - 4, 3), 0))
-        big_right_orig = CSFBigSite(n_external, self.ci_order, True,
-            fd_dyall, VectorUInt8(orb_sym[-n_external:]), max(min(self.verbose - 4, 3), 0))
+        if self.spin_adapted:
+            big_left_orig = CSFBigSite(n_inactive, self.ci_order, False,
+                fd_zero, VectorUInt8(orb_sym[:n_inactive]), max(min(self.verbose - 4, 3), 0))
+            big_right_orig = CSFBigSite(n_external, self.ci_order, True,
+                fd_zero, VectorUInt8(orb_sym[-n_external:]), max(min(self.verbose - 4, 3), 0))
+        else:
+            poccl = SCIFockBigSite.ras_space(False, n_inactive, *[self.ci_order] * 3, VectorInt([]))
+            poccr = SCIFockBigSite.ras_space(True, n_external, *[self.ci_order] * 3, VectorInt([]))
+            big_left_orig = SCIFockBigSite(n_orbs, n_inactive, False, fd_zero, VectorUInt8(orb_sym), poccl, self.verbose > 4)
+            big_right_orig = SCIFockBigSite(n_orbs, n_external, True, fd_zero, VectorUInt8(orb_sym), poccr, self.verbose > 4)
         big_left = SimplifiedBigSite(big_left_orig, RuleQC())
         big_right = SimplifiedBigSite(big_right_orig, RuleQC())
-        hm_dyall = HamiltonianQCBigSite(vacuum, n_orbs, VectorUInt8(orb_sym), fd_dyall,
+        hm_zero = HamiltonianQCBigSite(vacuum, n_orbs, VectorUInt8(orb_sym), fd_zero,
             big_left, big_right)
 
         # Left MPO
-        lmpo = MPOQC(hm_dyall, QCTypes.NC)
+        lmpo = MPOQC(hm_zero, QCTypes.NC)
         lmpo = SimplifiedMPO(lmpo, RuleQC(), True, True, OpNamesSet((OpNames.R, OpNames.RD)))
         lmpo.const_e -= self.e_tot
         lmpo = lmpo * -1
+        lmpo = IdentityAddedMPO(lmpo)
 
         # Right MPO
         rmpo = MPOQC(hamil, QCTypes.NC)
         rmpo = SimplifiedMPO(rmpo, NoTransposeRule(RuleQC()),
             True, True, OpNamesSet((OpNames.R, OpNames.RD)))
         rmpo.const_e -= self.e_tot
+        rmpo = IdentityAddedMPO(rmpo)
 
         # MPS
         mps.dot = 2
@@ -237,19 +302,25 @@ class NEVPT(lib.StreamObject):
 
         self.e_corr = linear.solve(len(bond_dims), mps.center == 0, sweep_tol)
 
-        lib.logger.note(self, 'E(NEVPT2) = %.16g  E_corr = %.16g', self.e_tot, self.e_corr)
+        lib.logger.note(self, 'E(%s2) = %.16g  E_corr = %.16g', self.theory.upper(), self.e_tot, self.e_corr)
 
         self.mps = bra
 
         if self.mp_order == 2:
             return self.e_corr, bra
-        
+
     def make_rdm1(self, state=None, norb=None, nelec=None):
         '''
         Spin-traced one-particle density matrix in MO basis.
         dm1[p,q] = <q_alpha^\dagger p_alpha> + <q_beta^\dagger p_beta>
         '''
 
+        from block2 import CG
+        if self.spin_adapted:
+            from block2.su2 import PDM1MPOQC, RuleQC, Expect, SimplifiedMPO, MovingEnvironment
+        else:
+            from block2.sz import PDM1MPOQC, RuleQC, Expect, SimplifiedMPO, MovingEnvironment
+
         if state is None:
             state = self.mps
             state.dot = 1
@@ -277,9 +348,15 @@ class NEVPT(lib.StreamObject):
 
         return dm.transpose((1, 0))
 
-def NEVPT2(*args, **kwargs):
+def UCNEVPT2(*args, **kwargs):
     kwargs['mp_order'] = 2
-    return NEVPT(*args, **kwargs)
+    kwargs['theory'] = 'nevpt'
+    return MRPT(*args, **kwargs)
+
+def UCMRREPT2(*args, **kwargs):
+    kwargs['mp_order'] = 2
+    kwargs['theory'] = 'mrrept'
+    return MRPT(*args, **kwargs)
 
 if __name__ == '__main__':
 
@@ -306,13 +383,13 @@ if __name__ == '__main__':
     dm1 = myci.make_rdm1()
 
     # DMRG NEVPT2
-    mypt = NEVPT2(myci)
+    mypt = UCNEVPT2(myci)
     mypt.dmrg_args['maxM'] = 1000
     mypt.dmrg_args['sweep_tol'] = 1E-16
     mypt = mypt.run()
 
     # DMRG NEVPT2 singles
-    mypt = NEVPT2(myci, ci_order=1)
+    mypt = UCNEVPT2(myci, ci_order=1)
     mypt.dmrg_args['maxM'] = 1000
     mypt.dmrg_args['sweep_tol'] = 1E-16
     mypt = mypt.run()
