@@ -132,7 +132,7 @@ struct CSRSparseMatrix : SparseMatrix<S, FL> {
         assert(info->n == other->info->n);
         deallocate();
         csr_data.resize(info->n);
-        if (ref && frame->minimal_memory_usage)
+        if (ref && frame_<FP>()->minimal_memory_usage)
             alloc = cother->alloc;
         for (int i = 0; i < info->n; i++)
             if (!ref)
@@ -147,7 +147,7 @@ struct CSRSparseMatrix : SparseMatrix<S, FL> {
                 // so double deallocate will not happen
                 // but if site tensor is persistent
                 // we should set alloc to nullptr to prevent double deallcoate
-                if (frame->minimal_memory_usage)
+                if (frame_<FP>()->minimal_memory_usage)
                     csr_data[i]->alloc = mat->alloc;
             }
     }
@@ -158,7 +158,7 @@ struct CSRSparseMatrix : SparseMatrix<S, FL> {
             dynamic_pointer_cast<CSRSparseMatrix>(other);
         deallocate();
         csr_data.resize(info->n);
-        if (ref && frame->minimal_memory_usage)
+        if (ref && frame_<FP>()->minimal_memory_usage)
             alloc = cother->alloc;
         for (int i = 0, k; i < other->info->n; i++)
             if ((k = info->find_state(other->info->quanta[i])) != -1) {
@@ -170,7 +170,7 @@ struct CSRSparseMatrix : SparseMatrix<S, FL> {
                     csr_data[k] = make_shared<GCSRMatrix<FL>>(
                         mat->m, mat->n, mat->nnz, mat->data, mat->rows,
                         mat->cols);
-                    if (frame->minimal_memory_usage)
+                    if (frame_<FP>()->minimal_memory_usage)
                         csr_data[k]->alloc = mat->alloc;
                 }
             }
