@@ -66,7 +66,7 @@ template <typename S, typename FL> struct FusedMPO : MPO<S, FL> {
         else {
             mpo->load_schemer();
             MPO<S, FL>::schemer =
-                frame->minimal_memory_usage
+                frame_<FP>()->minimal_memory_usage
                     ? make_shared<MPOSchemer<S>>(*mpo->schemer)
                     : mpo->schemer->copy();
             mpo->unload_schemer();
@@ -149,13 +149,13 @@ template <typename S, typename FL> struct FusedMPO : MPO<S, FL> {
                 }
         }
         // here main stack is not used
-        // but when frame->use_main_stack == false:
+        // but when frame_<FP>()->use_main_stack == false:
         // tf->left/right_contract will skip allocated matrices if alloc !=
         // nullptr
         for (auto &p : opt->ops) {
             shared_ptr<OpElement<S, FL>> op =
                 dynamic_pointer_cast<OpElement<S, FL>>(p.first);
-            if (frame->use_main_stack)
+            if (frame_<FP>()->use_main_stack)
                 p.second->alloc = d_alloc;
             p.second->info =
                 Partition<S, FL>::find_op_info(fused_op_infos, op->q_label);
